@@ -5,6 +5,7 @@
 extern crate fixedbitset;
 #[macro_use]
 extern crate itertools;
+extern crate ordermap;
 pub extern crate num_traits;
 
 mod astar;
@@ -29,13 +30,20 @@ pub use kuhn_munkres::*;
 pub use matrix::*;
 pub use topological_sort::*;
 
-use std::collections::{HashMap, VecDeque};
+use ordermap::OrderMap;
 use std::hash::Hash;
 
-fn reverse_path<N: Eq + Hash + Clone>(parents: &HashMap<N, N>, start: N) -> Vec<N> {
-    let mut path = std::iter::once(start).collect::<VecDeque<_>>();
-    while let Some(parent) = parents.get(&path[0]).cloned() {
-        path.push_front(parent);
-    }
-    path.into_iter().collect()
+fn reverse_path<N, V, F>(parents: &OrderMap<N, V>, parent: F, start: usize) -> Vec<N>
+where
+    N: Eq + Hash + Clone,
+    F: Fn(&V) -> usize,
+{
+    let path = itertools::unfold(start, |i| {
+        parents.get_index(*i).map(|(node, value)| {
+            *i = parent(value);
+            node
+        })
+    }).collect::<Vec<&N>>();
+
+    path.into_iter().rev().cloned().collect()
 }
